import unittest
import os

import openmdao.api as om
from openmdao.utils.assert_utils import assert_check_partials, assert_near_equal

from aviary.subsystems.mass.gasp_based.design_load import (DesignLoadGroup,
                                                           LoadFactors,
                                                           LoadParameters,
                                                           LoadSpeeds)
from aviary.variable_info.options import get_option_defaults
from aviary.variable_info.variables import Aircraft, Mission


# this is the large single aisle 1 V3 test case
class LoadSpeedsTestCase1(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(aviary_options=get_option_defaults()),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_airspeed"], 350, tol)  # bug fixed value
        assert_near_equal(self.prob["vel_c"], 350, tol)  # bug fixed value
        assert_near_equal(self.prob["max_maneuver_factor"], 2.5, tol)  # bug fixed value
        assert_near_equal(self.prob["min_dive_vel"], 420, tol)  # bug fixed value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=1e-7)


class LoadSpeedsTestCase2(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=0, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob["max_airspeed"], 346.75, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 306.15, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 3.8, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob["min_dive_vel"], 407.94, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=1e-7)


class LoadSpeedsTestCase3(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.LOADING_ABOVE_20, val=False, units='unitless')
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=1, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob["max_airspeed"], 401.6, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 315, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 4.4, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob["min_dive_vel"], 472.5, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=1e-7)


class LoadSpeedsTestCase4(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=2, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob["max_airspeed"], 320.17, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 294.27, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 6, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob["min_dive_vel"], 376.67, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=1e-6)


class LoadSpeedsTestCase5(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.LOADING_ABOVE_20, val=False, units='unitless')
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=4, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_airspeed"], 350, tol)  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 350, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 4, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["min_dive_vel"], 420, tol)  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=1e-7)


class LoadSpeedsTestCase6smooth(
    unittest.TestCase
):  # this is the large single aisle 1 V3 test case (LoadSpeedsTestCase1) with smooth functions
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(
                aviary_options=options
            ),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_airspeed"], 350, tol)  # bug fixed value
        assert_near_equal(self.prob["vel_c"], 350, tol)  # bug fixed value
        assert_near_equal(self.prob["max_maneuver_factor"], 2.5, tol)  # bug fixed value
        assert_near_equal(self.prob["min_dive_vel"], 420, tol)  # bug fixed value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=1e-7)


class LoadSpeedsTestCase7smooth(unittest.TestCase):  # TestCase2 with smooth functions
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=0, units='unitless')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(
                aviary_options=options
            ),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 2e-3
        assert_near_equal(
            self.prob["max_airspeed"], 346.75, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 306.15, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 3.8, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob["min_dive_vel"], 407.94, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-12, rtol=1e-12)


class LoadSpeedsTestCase8smooth(unittest.TestCase):  # TestCase3 with smooth functions
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.LOADING_ABOVE_20, val=False, units='unitless')
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=1, units='unitless')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(
                aviary_options=options,
            ),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob["max_airspeed"], 401.6, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 315, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 4.4, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob["min_dive_vel"], 472.5, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-5, rtol=1e-5)


class LoadSpeedsTestCase9smooth(unittest.TestCase):  # TestCase4 with smooth functions
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=2, units='unitless')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(
                aviary_options=options,
            ),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(
            self.prob["max_airspeed"], 320.17, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 294.27, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 6, tol
        )  # not actual GASP value
        assert_near_equal(
            self.prob["min_dive_vel"], 376.67, tol
        )  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-5, rtol=1e-5)


class LoadSpeedsTestCase10smooth(unittest.TestCase):  # TestCase5 with smooth functions
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Wing.LOADING_ABOVE_20, val=False, units='unitless')
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=4, units='unitless')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "speeds",
            LoadSpeeds(
                aviary_options=options,
            ),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # not actual bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_airspeed"], 350, tol)  # not actual GASP value
        assert_near_equal(self.prob["vel_c"], 350, tol)  # not actual GASP value
        assert_near_equal(
            self.prob["max_maneuver_factor"], 4, tol
        )  # not actual GASP value
        assert_near_equal(self.prob["min_dive_vel"], 420, tol)  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=1e-7)


# this is the large single aisle 1 V3 test case
class LoadParametersTestCase1(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=37500, units='ft')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "params", LoadParameters(aviary_options=options), promotes=["*"]
        )

        self.prob.model.set_input_defaults(
            "vel_c", val=350, units="kn"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "max_airspeed", val=350, units="kn"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 4e-4
        assert_near_equal(self.prob["max_mach"], 0.9, tol)  # bug fixed value
        assert_near_equal(self.prob["density_ratio"], 0.533, tol)  # bug fixed value
        assert_near_equal(self.prob["V9"], 350, tol)  # bug fixed value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-12, rtol=1e-12)


class LoadParametersTestCase2(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=2, units='unitless')
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=30000, units='ft')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "params", LoadParameters(aviary_options=options), promotes=["*"]
        )

        self.prob.model.set_input_defaults(
            "vel_c", val=350, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "max_airspeed", val=350, units="kn"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_mach"], 0.824, tol)  # not actual GASP value
        assert_near_equal(self.prob["density_ratio"], 0.682,
                          tol)  # not actual GASP value
        assert_near_equal(self.prob["V9"], 304.14, tol)  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=3e-8)


class LoadParametersTestCase3(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=4, units='unitless')
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=22000, units='ft')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "params", LoadParameters(aviary_options=options), promotes=["*"]
        )

        self.prob.model.set_input_defaults(
            "vel_c", val=350, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "max_airspeed", val=350, units="kn"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 2e-4
        assert_near_equal(self.prob["max_mach"], 0.7197, tol)  # not actual GASP value
        assert_near_equal(self.prob["density_ratio"], 0.6073,
                          tol)  # not actual GASP value
        assert_near_equal(self.prob["V9"], 304.14, tol)  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=6e-8)


# this is the large single aisle 1 V3 test case
class LoadParametersTestCase4smooth(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=37500, units='ft')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "params",
            LoadParameters(aviary_options=options,),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            "vel_c", val=350, units="kn"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "max_airspeed", val=350, units="kn"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 6e-4
        assert_near_equal(self.prob["max_mach"], 0.9, tol)  # bug fixed value
        assert_near_equal(self.prob["density_ratio"], 0.533, tol)  # bug fixed value
        assert_near_equal(self.prob["V9"], 350, tol)  # bug fixed value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-12, rtol=1e-12)


class LoadParametersTestCase5smooth(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=2, units='unitless')
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=30000, units='ft')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "params",
            LoadParameters(aviary_options=options,),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            "vel_c", val=350, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "max_airspeed", val=350, units="kn"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_mach"], 0.824, tol)  # not actual GASP value
        assert_near_equal(self.prob["density_ratio"], 0.682,
                          tol)  # not actual GASP value
        assert_near_equal(self.prob["V9"], 304.14, tol)  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=3e-8)


class LoadParametersTestCase6smooth(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.PART25_STRUCTURAL_CATEGORY,
                        val=4, units='unitless')
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=22000, units='ft')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "params",
            LoadParameters(aviary_options=options,),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            "vel_c", val=350, units="mi/h"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "max_airspeed", val=350, units="kn"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_mach"], 0.7197, tol)  # not actual GASP value
        assert_near_equal(self.prob["density_ratio"], 0.6073,
                          tol)  # not actual GASP value
        assert_near_equal(self.prob["V9"], 304.14, tol)  # not actual GASP value

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=6e-8)


# this is the large single aisle 1 V3 test case
class LoadFactorsTestCase1(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "factors", LoadFactors(aviary_options=get_option_defaults()), promotes=["*"]
        )

        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=126, units="lbf/ft**2"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "density_ratio", val=0.533, units="unitless"
        )  # bug fixed value
        self.prob.model.set_input_defaults("V9", val=350, units="kn")  # bug fixed value
        self.prob.model.set_input_defaults(
            "min_dive_vel", val=420, units="kn"
        )  # bug fixed value and original value
        self.prob.model.set_input_defaults(
            "max_maneuver_factor", val=2.5, units="unitless"
        )  # bug fixed value and original value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.AVERAGE_CHORD, val=12.71, units="ft"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Design.LIFT_CURVE_SLOPE, val=7.1765, units="1/rad"
        )  # bug fixed value and original value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        # bug fixed value
        assert_near_equal(self.prob[Aircraft.Wing.ULTIMATE_LOAD_FACTOR], 3.951, tol)

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=5e-7)


class LoadFactorsTestCase2(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.ULF_CALCULATED_FROM_MANEUVER,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "factors", LoadFactors(aviary_options=options), promotes=["*"]
        )

        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value
        self.prob.model.set_input_defaults(
            "density_ratio", val=0.5328, units="unitless"
        )  # may not be actual GASP value
        self.prob.model.set_input_defaults(
            "V9", val=304.14, units="kn"
        )  # may not be actual GASP value
        self.prob.model.set_input_defaults("min_dive_vel", val=420, units="kn")
        self.prob.model.set_input_defaults(
            "max_maneuver_factor", val=2.5, units="unitless")
        self.prob.model.set_input_defaults(
            Aircraft.Wing.AVERAGE_CHORD, val=12.615, units="ft"
        )
        self.prob.model.set_input_defaults(
            Aircraft.Design.LIFT_CURVE_SLOPE, val=7.1765, units="1/rad")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        # not actual GASP value
        assert_near_equal(self.prob[Aircraft.Wing.ULTIMATE_LOAD_FACTOR], 3.75, tol)

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-12, rtol=1e-12)


# this is the large single aisle 1 V3 test case
class LoadFactorsTestCase3smooth(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "factors",
            LoadFactors(aviary_options=options),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=126, units="lbf/ft**2"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            "density_ratio", val=0.533, units="unitless"
        )  # bug fixed value
        self.prob.model.set_input_defaults("V9", val=350, units="kn")  # bug fixed value
        self.prob.model.set_input_defaults(
            "min_dive_vel", val=420, units="kn"
        )  # bug fixed value and original value
        self.prob.model.set_input_defaults(
            "max_maneuver_factor", val=2.5, units="unitless"
        )  # bug fixed value and original value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.AVERAGE_CHORD, val=12.71, units="ft"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Design.LIFT_CURVE_SLOPE, val=7.1765, units="1/rad"
        )  # bug fixed value and original value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 4e-4
        # bug fixed value
        assert_near_equal(self.prob[Aircraft.Wing.ULTIMATE_LOAD_FACTOR], 3.951, tol)

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=5e-7)


class LoadFactorsTestCase4smooth(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Aircraft.Design.ULF_CALCULATED_FROM_MANEUVER,
                        val=True, units='unitless')

        self.prob = om.Problem()
        self.prob.model.add_subsystem(
            "factors",
            LoadFactors(aviary_options=options,),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=128, units="lbf/ft**2"
        )  # not actual bug fixed value
        self.prob.model.set_input_defaults(
            "density_ratio", val=0.5328, units="unitless"
        )  # may not be actual GASP value
        self.prob.model.set_input_defaults(
            "V9", val=304.14, units="kn"
        )  # may not be actual GASP value
        self.prob.model.set_input_defaults("min_dive_vel", val=420, units="kn")
        self.prob.model.set_input_defaults(
            "max_maneuver_factor", val=2.5, units="unitless")
        self.prob.model.set_input_defaults(
            Aircraft.Wing.AVERAGE_CHORD, val=12.615, units="ft"
        )
        self.prob.model.set_input_defaults(
            Aircraft.Design.LIFT_CURVE_SLOPE, val=7.1765, units="1/rad")

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        # not actual GASP value
        assert_near_equal(self.prob[Aircraft.Wing.ULTIMATE_LOAD_FACTOR], 3.75, tol)

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-8, rtol=1e-8)


# this is the large single aisle 1 V3 test case
class DesignLoadGroupTestCase1(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=37500, units='ft')

        self.prob = om.Problem()

        self.prob.model.add_subsystem(
            "Dload",
            DesignLoadGroup(
                aviary_options=options,
            ),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed and original value

        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=126, units="lbf/ft**2"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.AVERAGE_CHORD, val=12.71, units="ft"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 1e-4
        assert_near_equal(self.prob["max_mach"], 0.9, tol)  # bug fixed value
        # bug fixed value
        assert_near_equal(self.prob[Aircraft.Wing.ULTIMATE_LOAD_FACTOR], 3.75, tol)

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=2e-7)


# this is the large single aisle 1 V3 test case
class DesignLoadGroupTestCase2smooth(unittest.TestCase):
    def setUp(self):

        options = get_option_defaults()
        options.set_val(Mission.Design.CRUISE_ALTITUDE, val=37500, units='ft')
        options.set_val(Aircraft.Design.SMOOTH_MASS_DISCONTINUITIES,
                        val=True, units='unitless')

        self.prob = om.Problem()

        self.prob.model.add_subsystem(
            "Dload",
            DesignLoadGroup(
                aviary_options=options,
            ),
            promotes=["*"],
        )

        self.prob.model.set_input_defaults(
            Aircraft.Design.MAX_STRUCTURAL_SPEED, val=402.5, units="mi/h"
        )  # bug fixed and original value

        self.prob.model.set_input_defaults(
            Aircraft.Wing.LOADING, val=126, units="lbf/ft**2"
        )  # bug fixed value
        self.prob.model.set_input_defaults(
            Aircraft.Wing.AVERAGE_CHORD, val=12.71, units="ft"
        )  # bug fixed value

        self.prob.setup(check=False, force_alloc_complex=True)

    def test_case1(self):

        self.prob.run_model()

        tol = 6e-4
        assert_near_equal(self.prob["max_mach"], 0.9, tol)  # bug fixed value
        # bug fixed value
        assert_near_equal(self.prob[Aircraft.Wing.ULTIMATE_LOAD_FACTOR], 3.7397, tol)

        partial_data = self.prob.check_partials(out_stream=None, method="cs")
        assert_check_partials(partial_data, atol=1e-7, rtol=2e-7)


if __name__ == "__main__":
    unittest.main()



from copy import deepcopy
import unittest

from openmdao.utils.assert_utils import assert_near_equal
from openmdao.utils.testing_utils import use_tempdirs

from aviary.interface.methods_for_level2 import AviaryProblem
from aviary.interface.default_phase_info.height_energy import phase_info as ph_in_flops
from aviary.interface.default_phase_info.two_dof import phase_info as ph_in_gasp
from aviary.variable_info.variables import Aircraft, Mission


@use_tempdirs
class StaticGroupTest(unittest.TestCase):

    def test_post_mission_promotion(self):
        phase_info = deepcopy(ph_in_flops)

        prob = AviaryProblem()

        csv_path = "models/test_aircraft/aircraft_for_bench_GwFm.csv"

        prob.load_inputs(csv_path, phase_info)
        prob.check_and_preprocess_inputs()

        prob.aviary_inputs.set_val(
            Aircraft.Design.RESERVE_FUEL_ADDITIONAL, 10000.0, units='lbm')

        prob.add_pre_mission_systems()
        prob.add_phases()
        prob.add_post_mission_systems()

        prob.link_phases()

        prob.add_design_variables()
        prob.add_objective(objective_type="mass", ref=-1e5)

        prob.setup()
        prob.set_initial_guesses()

        prob.run_model()

        fuel_burned = prob.model.get_val('fuel_burned', units='lbm')
        total_fuel = prob.model.get_val(Mission.Summary.TOTAL_FUEL_MASS, units='lbm')

        assert_near_equal(total_fuel - fuel_burned, 10000.0, 1e-3)

    def test_gasp_relative_reserve(self):
        phase_info = deepcopy(ph_in_gasp)

        prob = AviaryProblem()

        csv_path = "models/small_single_aisle/small_single_aisle_GwGm.csv"

        prob.load_inputs(csv_path, phase_info)
        prob.check_and_preprocess_inputs()

        prob.aviary_inputs.set_val(Mission.Summary.GROSS_MASS, 140000.0, units='lbm')

        prob.add_pre_mission_systems()
        prob.add_phases()
        prob.add_post_mission_systems()

        prob.link_phases()

        prob.add_design_variables()
        prob.add_objective(objective_type="mass", ref=-1e5)

        prob.setup()
        prob.set_initial_guesses()

        prob.run_model()

        res_frac = prob.aviary_inputs.get_val(
            Aircraft.Design.RESERVE_FUEL_FRACTION, units='unitless')
        td_mass = prob.model.get_val(Mission.Landing.TOUCHDOWN_MASS, units='lbm')
        reserve = prob.model.get_val(Mission.Design.RESERVE_FUEL, units='lbm')
        assert_near_equal(reserve, res_frac * (140000.0 - td_mass), 1e-3)


if __name__ == '__main__':
    unittest.main()



import unittest
import os

import openmdao.api as om
from openmdao.utils.assert_utils import (assert_check_partials,
                                         assert_near_equal)

from aviary.subsystems.aerodynamics.gasp_based.flaps_model.basic_calculations import \
    BasicFlapsCalculations
from aviary.variable_info.variables import Aircraft

"""
All data is from validation files using standalone flaps model
"""


class BasicFlapsCalculationsTestCase(unittest.TestCase):
    def setUp(self):

        self.prob = om.Problem(model=om.Group())

        self.prob.model.add_subsystem('BC', BasicFlapsCalculations(), promotes=['*'])

        self.prob.setup()

        # initial conditions
        self.prob.set_val(Aircraft.Wing.SWEEP, 25.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ASPECT_RATIO, 10.13)
        self.prob.set_val(Aircraft.Wing.FLAP_CHORD_RATIO, 0.3)
        self.prob.set_val(Aircraft.Wing.TAPER_RATIO, 0.33)
        self.prob.set_val(Aircraft.Wing.CENTER_CHORD, 17.48974, units="ft")
        self.prob.set_val(Aircraft.Fuselage.AVG_DIAMETER, 13.1, units="ft")
        self.prob.set_val(Aircraft.Wing.THICKNESS_TO_CHORD_ROOT, 0.15)
        self.prob.set_val(Aircraft.Wing.SPAN, 117.8, units="ft")
        self.prob.set_val(Aircraft.Wing.SLAT_CHORD_RATIO, 0.15)
        self.prob.set_val("slat_defl", 10.0, units="deg")
        self.prob.set_val(Aircraft.Wing.OPTIMUM_SLAT_DEFLECTION, 20.0, units="deg")
        self.prob.set_val(Aircraft.Wing.ROOT_CHORD, 16.41, units="ft")
        self.prob.set_val(Aircraft.Fuselage.LENGTH, 129.4, units="ft")
        self.prob.set_val(Aircraft.Wing.LEADING_EDGE_SWEEP, 0.47639, units="rad")

    def test_case(self):

        self.prob.run_model()
        tol = 2.1e-4
        print()

        reg_data = 0.74444
        ans = self.prob["VLAM8"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.93578
        ans = self.prob["VDEL4"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.90761
        ans = self.prob["VDEL5"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.9975
        ans = self.prob["VLAM9"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.5
        ans = self.prob["slat_defl_ratio"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.89761
        ans = self.prob[Aircraft.Wing.SLAT_SPAN_RATIO]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.09239
        ans = self.prob["body_to_span_ratio"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.12679
        ans = self.prob["chord_to_body_ratio"]
        assert_near_equal(ans, reg_data, tol)

        reg_data = 0.79208
        ans = self.prob["VLAM12"]
        assert_near_equal(ans, reg_data, tol)

        data = self.prob.check_partials(out_stream=None, method="fd")
        assert_check_partials(data, atol=1e-6, rtol=4e-6)


if __name__ == "__main__":
    unittest.main()



import openmdao.api as om

from aviary.constants import GRAV_ENGLISH_LBM
from aviary.utils.aviary_values import AviaryValues
from aviary.variable_info.functions import add_aviary_input, add_aviary_output
from aviary.variable_info.variables import Aircraft


class TransportAPUMass(om.ExplicitComponent):
    """
    Calculates the mass of the auxiliary power unit. The methodology is based
    on the FLOPS weight equations, modified to output mass instead of weight.
    """

    def initialize(self):
        self.options.declare(
            'aviary_options', types=AviaryValues,
            desc='collection of Aircraft/Mission specific options')

    def setup(self):
        add_aviary_input(self, Aircraft.APU.MASS_SCALER, val=1.0)

        add_aviary_input(self, Aircraft.Fuselage.PLANFORM_AREA, val=0.0)

        add_aviary_output(self, Aircraft.APU.MASS, val=0.0)

    def setup_partials(self):
        self.declare_partials('*', '*')

    def compute(self, inputs, outputs):
        aviary_options: AviaryValues = self.options['aviary_options']
        pax = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_PASSENGERS, units='unitless')
        scaler = inputs[Aircraft.APU.MASS_SCALER]
        planform = inputs[Aircraft.Fuselage.PLANFORM_AREA]

        outputs[Aircraft.APU.MASS] = (
            54.0 * planform ** 0.3 + 5.4 * pax ** 0.9) * scaler / GRAV_ENGLISH_LBM

    def compute_partials(self, inputs, J):
        aviary_options: AviaryValues = self.options['aviary_options']
        pax = aviary_options.get_val(
            Aircraft.CrewPayload.NUM_PASSENGERS, units='unitless')
        scaler = inputs[Aircraft.APU.MASS_SCALER]
        planform = inputs[Aircraft.Fuselage.PLANFORM_AREA]

        J[Aircraft.APU.MASS, Aircraft.APU.MASS_SCALER] = (
            54.0 * planform ** 0.3 + 5.4 * pax ** 0.9) / GRAV_ENGLISH_LBM
        J[Aircraft.APU.MASS, Aircraft.Fuselage.PLANFORM_AREA] = \
            16.2 * planform ** -0.7 * scaler / GRAV_ENGLISH_LBM
